"""Tests for vmware_vks.connection (unit, no real vCenter)."""
from unittest.mock import MagicMock, patch
from vmware_vks.config import AppConfig, TargetConfig
from vmware_vks.connection import ConnectionManager


def _make_config():
    target = TargetConfig(name="vc1", host="vc.example.com", username="admin@vsphere.local")
    return AppConfig(targets=(target,))


def test_connection_manager_list_targets():
    mgr = ConnectionManager(_make_config())
    assert mgr.list_targets() == ["vc1"]


def test_connection_manager_connect_reuses_session(monkeypatch):
    monkeypatch.setenv("VMWARE_VKS_VC1_PASSWORD", "pw")
    mgr = ConnectionManager(_make_config())

    mock_si = MagicMock()
    mock_si.content.sessionManager.currentSession = "active"

    with patch.object(mgr, "_create_connection", return_value=mock_si) as mock_create:
        si1 = mgr.connect("vc1")
        si2 = mgr.connect("vc1")
        assert si1 is si2
        mock_create.assert_called_once()



"""Tests for TKC ops (unit, no real vCenter/K8s)."""
from unittest.mock import MagicMock, patch
import pytest
from vmware_vks.ops.tkc import (
    generate_tkc_yaml,
    list_tkc_clusters,
    get_tkc_available_versions,
    delete_tkc_cluster,
)


def test_generate_tkc_yaml_contains_cluster_name():
    yaml_str = generate_tkc_yaml(
        name="my-cluster", namespace="dev",
        k8s_version="v1.28.4+vmware.1", vm_class="best-effort-large",
        control_plane_count=1, worker_count=3, storage_class="vsphere-storage",
    )
    assert "my-cluster" in yaml_str
    assert "best-effort-large" in yaml_str
    assert "Cluster" in yaml_str


def test_generate_tkc_yaml_invalid_worker_count():
    with pytest.raises(ValueError, match="worker_count"):
        generate_tkc_yaml(
            name="bad", namespace="dev", k8s_version="v1.28.4+vmware.1",
            vm_class="best-effort-large", control_plane_count=1,
            worker_count=0, storage_class="vsphere-storage",
        )


def test_generate_tkc_yaml_invalid_control_plane():
    with pytest.raises(ValueError, match="control_plane_count"):
        generate_tkc_yaml(
            name="bad", namespace="dev", k8s_version="v1.28.4+vmware.1",
            vm_class="best-effort-large", control_plane_count=2,
            worker_count=3, storage_class="vsphere-storage",
        )


def test_list_tkc_clusters_empty():
    si = MagicMock()
    mock_api = MagicMock()
    mock_api.list_namespaced_custom_object.return_value = {"items": []}
    with patch("vmware_vks.ops.tkc._get_custom_objects_api", return_value=mock_api):
        result = list_tkc_clusters(si, namespace="dev")
    assert result["clusters"] == []
    assert result["total"] == 0


def test_delete_tkc_cluster_requires_confirmed():
    si = MagicMock()
    with pytest.raises(ValueError, match="confirmed=True"):
        delete_tkc_cluster(si, "my-cluster", "dev", confirmed=False)


def test_delete_tkc_cluster_dry_run():
    si = MagicMock()
    with patch("vmware_vks.ops.tkc._check_running_workloads", return_value=[]):
        result = delete_tkc_cluster(si, "my-cluster", "dev", confirmed=True, dry_run=True)
    assert result["dry_run"] is True


def test_delete_tkc_cluster_rejects_with_workloads():
    si = MagicMock()
    workloads = [{"kind": "Deployment", "name": "app", "namespace": "default"}]
    with patch("vmware_vks.ops.tkc._check_running_workloads", return_value=workloads):
        with pytest.raises(RuntimeError, match="Deployment/app"):
            delete_tkc_cluster(si, "my-cluster", "dev", confirmed=True, dry_run=False, force=False)



"""Layer 2: kubernetes Python client connection to Supervisor K8s API endpoint.

Kubeconfig is built from the active pyVmomi session and vCenter REST info.
Used for TKC CR lifecycle (create/get/delete/scale/upgrade).
"""
from __future__ import annotations

import logging
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance

_log = logging.getLogger("vmware-vks.k8s_connection")


def _vcenter_host(si: ServiceInstance) -> str:
    return si._stub.host.split(":")[0]


def get_supervisor_kubeconfig_str(si: ServiceInstance, namespace: str) -> str:
    """Build a kubeconfig YAML string for the Supervisor API endpoint.

    Uses the vCenter session token as bearer token and queries the
    namespace-management API for the cluster API endpoint.
    """
    from vmware_vks.ops.supervisor import _rest_get
    import yaml as _yaml

    # Get supervisor clusters to find API endpoint
    try:
        clusters = _rest_get(si, "/vcenter/namespace-management/clusters")
        running = [c for c in clusters if c.get("config_status") == "RUNNING"]
        if not running:
            raise RuntimeError("No running Supervisor clusters found.")
        # Use api_server_cluster_endpoint from the first running cluster
        cluster_data = _rest_get(
            si,
            f"/vcenter/namespace-management/clusters/{running[0]['cluster']}"
        )
        api_endpoint = cluster_data.get("api_server_cluster_endpoint", "")
        if not api_endpoint:
            raise RuntimeError("Could not determine Supervisor API endpoint.")
    except Exception as e:
        raise RuntimeError(
            f"Could not retrieve Supervisor API endpoint: {e}. "
            "Ensure Workload Management is enabled."
        ) from e

    token = si.content.sessionManager.currentSession.key

    kubeconfig = {
        "apiVersion": "v1",
        "kind": "Config",
        "clusters": [{
            "name": "supervisor",
            "cluster": {
                "server": f"https://{api_endpoint}",
                "insecure-skip-tls-verify": True,
            }
        }],
        "users": [{"name": "vsphere-user", "user": {"token": token}}],
        "contexts": [{
            "name": "supervisor-context",
            "context": {
                "cluster": "supervisor",
                "user": "vsphere-user",
                "namespace": namespace,
            }
        }],
        "current-context": "supervisor-context",
    }
    return _yaml.dump(kubeconfig)


def get_k8s_client(si: ServiceInstance, namespace: str):
    """Get a kubernetes ApiClient connected to the Supervisor namespace.

    Returns a kubernetes.client.ApiClient instance.
    """
    import kubernetes as k8s

    kubeconfig_str = get_supervisor_kubeconfig_str(si, namespace)

    with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as f:
        f.write(kubeconfig_str)
        tmpfile = f.name

    try:
        cfg = k8s.config.load_kube_config(config_file=tmpfile)
        return k8s.client.ApiClient(configuration=cfg)
    finally:
        Path(tmpfile).unlink(missing_ok=True)



"""Connection management for vCenter (Layer 1: pyVmomi).

Layer 2 (kubernetes client) is handled separately in vmware_vks.k8s_connection.
"""
from __future__ import annotations

import atexit
import ssl
from typing import TYPE_CHECKING

from pyVmomi import vmodl
from vmware_vks.config import AppConfig, TargetConfig, load_config

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance


class ConnectionManager:
    """Manages pyVmomi connections to multiple vCenter targets."""

    def __init__(self, config: AppConfig) -> None:
        self._config = config
        self._connections: dict[str, ServiceInstance] = {}

    @classmethod
    def from_config(cls, config: AppConfig | None = None) -> ConnectionManager:
        cfg = config or load_config()
        return cls(cfg)

    def connect(self, target_name: str | None = None) -> ServiceInstance:
        target = (
            self._config.get_target(target_name)
            if target_name
            else self._config.default_target
        )
        if target.name in self._connections:
            si = self._connections[target.name]
            try:
                _ = si.content.sessionManager.currentSession
                return si
            except (vmodl.fault.NotAuthenticated, Exception):
                del self._connections[target.name]
        si = self._create_connection(target)
        self._connections[target.name] = si
        return si

    def list_targets(self) -> list[str]:
        return [t.name for t in self._config.targets]

    @staticmethod
    def _create_connection(target: TargetConfig) -> ServiceInstance:
        from pyVim.connect import Disconnect, SmartConnect

        context = None
        if not target.verify_ssl:
            context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE

        si = SmartConnect(
            host=target.host,
            user=target.username,
            pwd=target.password,
            port=target.port,
            sslContext=context,
            disableSslCertValidation=not target.verify_ssl,
        )
        atexit.register(Disconnect, si)
        return si



"""Namespace lifecycle operations.

All write operations require confirmed=True.
delete_namespace has an additional guard: rejects if TKC clusters exist inside.
"""
from __future__ import annotations

import json
import logging
import ssl
import urllib.error
import urllib.request
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance

from vmware_vks.ops.supervisor import _rest_get

_log = logging.getLogger("vmware-vks.ops.namespace")


def _rest_post(si: ServiceInstance, path: str, body: dict) -> Any:
    host = si._stub.host.split(":")[0]
    session_id = si.content.sessionManager.currentSession.key
    url = f"https://{host}/api{path}"
    data = json.dumps(body).encode()
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
    req = urllib.request.Request(
        url, data=data,
        headers={"vmware-api-session-id": session_id, "Content-Type": "application/json"},
        method="POST",
    )
    try:
        with urllib.request.urlopen(req, context=ctx) as resp:  # nosec B310
            body_resp = resp.read()
            return json.loads(body_resp) if body_resp else {}
    except urllib.error.HTTPError as e:
        raise RuntimeError(f"REST POST {path} failed ({e.code}): {e.read().decode()}") from e


def _rest_patch(si: ServiceInstance, path: str, body: dict) -> Any:
    host = si._stub.host.split(":")[0]
    session_id = si.content.sessionManager.currentSession.key
    url = f"https://{host}/api{path}"
    data = json.dumps(body).encode()
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
    req = urllib.request.Request(
        url, data=data,
        headers={"vmware-api-session-id": session_id, "Content-Type": "application/json"},
        method="PATCH",
    )
    try:
        with urllib.request.urlopen(req, context=ctx) as resp:  # nosec B310
            body_resp = resp.read()
            return json.loads(body_resp) if body_resp else {}
    except urllib.error.HTTPError as e:
        raise RuntimeError(f"REST PATCH {path} failed ({e.code}): {e.read().decode()}") from e


def _rest_delete(si: ServiceInstance, path: str) -> None:
    host = si._stub.host.split(":")[0]
    session_id = si.content.sessionManager.currentSession.key
    url = f"https://{host}/api{path}"
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
    req = urllib.request.Request(
        url, headers={"vmware-api-session-id": session_id}, method="DELETE",
    )
    try:
        with urllib.request.urlopen(req, context=ctx) as _:  # nosec B310
            pass
    except urllib.error.HTTPError as e:
        raise RuntimeError(f"REST DELETE {path} failed ({e.code}): {e.read().decode()}") from e


def _list_tkc_in_namespace(si: ServiceInstance, namespace: str) -> list[str]:
    """Return TKC cluster names in namespace. Used as delete guard."""
    try:
        from vmware_vks.ops.tkc import list_tkc_clusters
        result = list_tkc_clusters(si, namespace=namespace)
        return [c["name"] for c in result.get("clusters", [])]
    except Exception:
        return []


def list_namespaces(si: ServiceInstance) -> list[dict]:
    """List all vSphere Namespaces."""
    data = _rest_get(si, "/vcenter/namespaces/instances")
    return [
        {
            "namespace": item.get("namespace"),
            "config_status": item.get("config_status"),
            "description": item.get("description", ""),
        }
        for item in (data if isinstance(data, list) else [])
    ]


def get_namespace(si: ServiceInstance, name: str) -> dict:
    """Get detailed info for a single namespace."""
    return _rest_get(si, f"/vcenter/namespaces/instances/{name}")


def create_namespace(
    si: ServiceInstance,
    name: str,
    cluster_id: str,
    storage_policy: str,
    cpu_limit: int | None = None,
    memory_limit_mib: int | None = None,
    description: str = "",
    dry_run: bool = False,
) -> dict:
    """Create a vSphere Namespace."""
    spec: dict = {
        "namespace": name,
        "cluster": cluster_id,
        "description": description,
        "storage_specs": [{"policy": storage_policy}],
    }
    resource_spec: dict = {}
    if cpu_limit is not None:
        resource_spec["cpu_limit"] = cpu_limit
    if memory_limit_mib is not None:
        resource_spec["memory_limit"] = memory_limit_mib
    if resource_spec:
        spec["resource_spec"] = resource_spec

    if dry_run:
        return {"dry_run": True, "spec": spec, "action": "create_namespace"}

    _rest_post(si, "/vcenter/namespaces/instances", spec)
    return {"namespace": name, "status": "created", "cluster": cluster_id}


def update_namespace(
    si: ServiceInstance,
    name: str,
    cpu_limit: int | None = None,
    memory_limit_mib: int | None = None,
    storage_policy: str | None = None,
) -> dict:
    """Update namespace resource quotas or storage policy."""
    spec: dict = {}
    resource_spec: dict = {}
    if cpu_limit is not None:
        resource_spec["cpu_limit"] = cpu_limit
    if memory_limit_mib is not None:
        resource_spec["memory_limit"] = memory_limit_mib
    if resource_spec:
        spec["resource_spec"] = resource_spec
    if storage_policy:
        spec["storage_specs"] = [{"policy": storage_policy}]
    if not spec:
        return {"namespace": name, "status": "no_changes"}
    _rest_patch(si, f"/vcenter/namespaces/instances/{name}", spec)
    return {"namespace": name, "status": "updated"}


def delete_namespace(
    si: ServiceInstance,
    name: str,
    confirmed: bool = False,
    dry_run: bool = False,
) -> dict:
    """Delete a vSphere Namespace.

    Guards: confirmed=True required; rejects if TKC clusters exist.
    """
    if not confirmed:
        raise ValueError(
            f"confirmed=True required to delete namespace '{name}'."
        )

    tkc_clusters = _list_tkc_in_namespace(si, name)
    if tkc_clusters:
        raise RuntimeError(
            f"Cannot delete namespace '{name}': "
            f"TKC clusters still exist: {', '.join(tkc_clusters)}. "
            "Delete all TKC clusters first."
        )

    if dry_run:
        return {
            "dry_run": True,
            "action": "delete_namespace",
            "namespace": name,
            "warning": "This will permanently delete the namespace.",
        }

    _rest_delete(si, f"/vcenter/namespaces/instances/{name}")
    return {"namespace": name, "status": "deleted"}


def list_vm_classes(si: ServiceInstance) -> list[dict]:
    """List available VM classes for TKC node sizing."""
    data = _rest_get(si, "/vcenter/namespace-management/virtual-machine-classes")
    return [
        {
            "id": item.get("id"),
            "cpu_count": item.get("cpu_count"),
            "memory_mib": item.get("memory_mib"),
            "gpu_count": item.get("gpu_count", 0),
        }
        for item in (data if isinstance(data, list) else [])
    ]



"""Harbor registry info (read-only)."""
from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance

from vmware_vks.ops.supervisor import _rest_get


def get_harbor_info(si: ServiceInstance) -> dict:
    """Get embedded Harbor registry info."""
    try:
        data = _rest_get(si, "/vcenter/content/registries/harbor")
        registries = data if isinstance(data, list) else [data]
        return {
            "registries": [
                {
                    "id": r.get("id"),
                    "url": r.get("ui_access_url"),
                    "storage_used_mb": r.get("storage_used_MB"),
                    "status": r.get("health", {}).get("status"),
                }
                for r in registries
            ]
        }
    except Exception as e:
        return {"error": str(e), "hint": "Harbor may not be enabled on this Supervisor"}



"""Tests for storage ops."""
from unittest.mock import MagicMock, patch, call
import pytest
from vmware_vks.ops.storage import list_namespace_storage_usage


def _mock_si():
    si = MagicMock()
    si.content.about.version = "8.0.2"
    si._stub.host = "vcenter.example.com:443"
    si.content.sessionManager.currentSession.key = "session-123"
    return si


def _make_pvc(name: str, namespace: str, phase: str, capacity_gb: str, sc: str):
    pvc = MagicMock()
    pvc.metadata.name = name
    pvc.metadata.namespace = namespace
    pvc.status.phase = phase
    pvc.status.capacity = {"storage": capacity_gb}
    pvc.spec.storage_class_name = sc
    return pvc


def test_list_namespace_storage_usage_returns_list():
    si = _mock_si()
    mock_api_client = MagicMock()
    mock_core_api = MagicMock()

    pvc1 = _make_pvc("data-vol", "dev", "Bound", "10Gi", "vsphere-storage")
    pvc2 = _make_pvc("log-vol", "dev", "Bound", "5Gi", "vsphere-storage")
    mock_core_api.list_namespaced_persistent_volume_claim.return_value = MagicMock(items=[pvc1, pvc2])

    with patch("vmware_vks.k8s_connection.get_k8s_client", return_value=mock_api_client):
        with patch("kubernetes.client.CoreV1Api", return_value=mock_core_api):
            result = list_namespace_storage_usage(si, "dev")

    assert result["namespace"] == "dev"
    assert result["pvc_count"] == 2
    assert len(result["pvcs"]) == 2
    assert result["pvcs"][0]["name"] == "data-vol"
    assert result["pvcs"][1]["capacity"] == "5Gi"


def test_list_namespace_storage_usage_uses_correct_namespace():
    """Regression test: verify namespace parameter is passed through to the k8s API call."""
    si = _mock_si()
    mock_api_client = MagicMock()
    mock_core_api = MagicMock()
    mock_core_api.list_namespaced_persistent_volume_claim.return_value = MagicMock(items=[])

    with patch("vmware_vks.k8s_connection.get_k8s_client", return_value=mock_api_client) as mock_get_client:
        with patch("kubernetes.client.CoreV1Api", return_value=mock_core_api):
            list_namespace_storage_usage(si, "production")

    # Verify the namespace was passed to both the k8s client and the PVC list call
    mock_get_client.assert_called_once_with(si, "production")
    mock_core_api.list_namespaced_persistent_volume_claim.assert_called_once_with(namespace="production")


def test_list_namespace_storage_usage_empty():
    si = _mock_si()
    mock_api_client = MagicMock()
    mock_core_api = MagicMock()
    mock_core_api.list_namespaced_persistent_volume_claim.return_value = MagicMock(items=[])

    with patch("vmware_vks.k8s_connection.get_k8s_client", return_value=mock_api_client):
        with patch("kubernetes.client.CoreV1Api", return_value=mock_core_api):
            result = list_namespace_storage_usage(si, "empty-ns")

    assert result["namespace"] == "empty-ns"
    assert result["pvc_count"] == 0
    assert result["pvcs"] == []



"""Tests for vmware_vks.config."""
import os
from pathlib import Path
import pytest
from vmware_vks.config import AppConfig, TargetConfig, load_config


def test_target_config_password_from_env(monkeypatch):
    monkeypatch.setenv("VMWARE_VKS_VCENTER01_PASSWORD", "secret123")
    t = TargetConfig(name="vcenter01", host="vc.example.com", username="admin@vsphere.local")
    assert t.password == "secret123"


def test_target_config_password_missing_raises(monkeypatch):
    monkeypatch.delenv("VMWARE_VKS_VCENTER01_PASSWORD", raising=False)
    t = TargetConfig(name="vcenter01", host="vc.example.com", username="admin@vsphere.local")
    with pytest.raises(OSError, match="VMWARE_VKS_VCENTER01_PASSWORD"):
        _ = t.password


def test_load_config(tmp_path):
    cfg_file = tmp_path / "config.yaml"
    cfg_file.write_text(
        "targets:\n"
        "  - name: vc1\n"
        "    host: vc.example.com\n"
        "    username: admin@vsphere.local\n"
    )
    config = load_config(cfg_file)
    assert len(config.targets) == 1
    assert config.targets[0].name == "vc1"
    assert config.default_target.host == "vc.example.com"


def test_load_config_missing_file():
    with pytest.raises(FileNotFoundError):
        load_config(Path("/nonexistent/config.yaml"))


def test_get_target_not_found(tmp_path):
    cfg_file = tmp_path / "config.yaml"
    cfg_file.write_text(
        "targets:\n"
        "  - name: vc1\n"
        "    host: vc.example.com\n"
        "    username: admin@vsphere.local\n"
    )
    config = load_config(cfg_file)
    with pytest.raises(KeyError, match="vc2"):
        config.get_target("vc2")



"""TanzuKubernetesCluster (TKC) lifecycle operations.

Uses cluster.x-k8s.io/v1beta1 API (vSphere 8.x).
All cluster operations go through the Supervisor K8s API endpoint (Layer 2).

Safety:
- delete_tkc_cluster rejects if Deployments/StatefulSets/DaemonSets are running
- create_tkc_cluster defaults to dry_run=True (returns YAML plan)
"""
from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING

import yaml

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance

_log = logging.getLogger("vmware-vks.ops.tkc")

_TKC_GROUP = "cluster.x-k8s.io"
_TKC_VERSION = "v1beta1"
_TKC_PLURAL = "clusters"


def _get_custom_objects_api(si: ServiceInstance, namespace: str):
    """Get kubernetes CustomObjectsApi connected to Supervisor namespace."""
    import kubernetes as k8s
    from vmware_vks.k8s_connection import get_k8s_client
    api_client = get_k8s_client(si, namespace)
    return k8s.client.CustomObjectsApi(api_client)


def generate_tkc_yaml(
    name: str,
    namespace: str,
    k8s_version: str,
    vm_class: str,
    control_plane_count: int,
    worker_count: int,
    storage_class: str,
) -> str:
    """Generate TKC cluster YAML (cluster.x-k8s.io/v1beta1 for vSphere 8.x)."""
    if worker_count < 1:
        raise ValueError(f"worker_count must be >= 1, got {worker_count}")
    if control_plane_count not in (1, 3):
        raise ValueError(f"control_plane_count must be 1 or 3, got {control_plane_count}")

    manifest = {
        "apiVersion": f"{_TKC_GROUP}/{_TKC_VERSION}",
        "kind": "Cluster",
        "metadata": {"name": name, "namespace": namespace},
        "spec": {
            "clusterNetwork": {
                "pods": {"cidrBlocks": ["192.168.0.0/16"]},
                "services": {"cidrBlocks": ["10.96.0.0/12"]},
            },
            "topology": {
                "class": "tanzukubernetescluster",
                "version": k8s_version,
                "controlPlane": {
                    "replicas": control_plane_count,
                    "metadata": {},
                    "nodeDrainTimeout": "60s",
                },
                "workers": {
                    "machineDeployments": [{
                        "class": "node-pool",
                        "name": "worker-pool",
                        "replicas": worker_count,
                        "metadata": {},
                        "nodeDrainTimeout": "60s",
                    }]
                },
                "variables": [
                    {"name": "vmClass", "value": vm_class},
                    {"name": "storageClass", "value": storage_class},
                ],
            },
        },
    }
    return yaml.dump(manifest, default_flow_style=False)


def list_tkc_clusters(si: ServiceInstance, namespace: str | None = None) -> dict:
    """List TKC clusters in a namespace (or all namespaces)."""
    ns = namespace or "default"
    api = _get_custom_objects_api(si, ns)
    if namespace:
        raw = api.list_namespaced_custom_object(
            group=_TKC_GROUP, version=_TKC_VERSION,
            namespace=namespace, plural=_TKC_PLURAL,
        )
    else:
        raw = api.list_cluster_custom_object(
            group=_TKC_GROUP, version=_TKC_VERSION, plural=_TKC_PLURAL,
        )
    items = raw.get("items", [])
    clusters = [
        {
            "name": item["metadata"]["name"],
            "namespace": item["metadata"]["namespace"],
            "phase": item.get("status", {}).get("phase", "Unknown"),
            "k8s_version": item["spec"]["topology"].get("version", ""),
        }
        for item in items
    ]
    return {"total": len(clusters), "clusters": clusters}


def get_tkc_cluster(si: ServiceInstance, name: str, namespace: str) -> dict:
    """Get detailed TKC cluster info."""
    api = _get_custom_objects_api(si, namespace)
    raw = api.get_namespaced_custom_object(
        group=_TKC_GROUP, version=_TKC_VERSION,
        namespace=namespace, plural=_TKC_PLURAL, name=name,
    )
    status = raw.get("status", {})
    return {
        "name": name,
        "namespace": namespace,
        "phase": status.get("phase"),
        "k8s_version": raw["spec"]["topology"].get("version"),
        "control_plane_replicas": raw["spec"]["topology"]["controlPlane"].get("replicas"),
        "worker_replicas": raw["spec"]["topology"]["workers"]["machineDeployments"][0].get("replicas"),
        "conditions": status.get("conditions", []),
        "infrastructure_ready": status.get("infrastructureReady", False),
        "control_plane_ready": status.get("controlPlaneReady", False),
    }


def get_tkc_available_versions(si: ServiceInstance, namespace: str) -> dict:
    """List K8s versions available for TKC clusters on this Supervisor."""
    import kubernetes as k8s
    from vmware_vks.k8s_connection import get_k8s_client

    api_client = get_k8s_client(si, namespace)
    custom_api = k8s.client.CustomObjectsApi(api_client)

    try:
        raw = custom_api.list_cluster_custom_object(
            group="run.tanzu.vmware.com",
            version="v1alpha3",
            plural="tanzukubernetesreleases",
        )
        versions = [
            {
                "name": item["metadata"]["name"],
                "version": item["spec"].get("version", item["metadata"]["name"]),
            }
            for item in raw.get("items", [])
        ]
        return {"versions": sorted(versions, key=lambda x: x["version"], reverse=True)}
    except Exception as e:
        return {"versions": [], "error": str(e),
                "hint": "TanzuKubernetesRelease API may not be available on this Supervisor"}


def create_tkc_cluster(
    si: ServiceInstance,
    name: str,
    namespace: str,
    k8s_version: str,
    vm_class: str,
    control_plane_count: int = 1,
    worker_count: int = 3,
    storage_class: str = "vsphere-storage",
    dry_run: bool = True,
) -> dict:
    """Create a TKC cluster. dry_run=True returns YAML plan without applying."""
    yaml_str = generate_tkc_yaml(
        name=name, namespace=namespace, k8s_version=k8s_version,
        vm_class=vm_class, control_plane_count=control_plane_count,
        worker_count=worker_count, storage_class=storage_class,
    )

    if dry_run:
        return {
            "dry_run": True,
            "action": "create_tkc_cluster",
            "name": name,
            "namespace": namespace,
            "yaml": yaml_str,
            "hint": "Set dry_run=False to apply this manifest.",
        }

    manifest = yaml.safe_load(yaml_str)
    api = _get_custom_objects_api(si, namespace)
    api.create_namespaced_custom_object(
        group=_TKC_GROUP, version=_TKC_VERSION,
        namespace=namespace, plural=_TKC_PLURAL, body=manifest,
    )
    return {"name": name, "namespace": namespace, "status": "creating", "yaml": yaml_str}


def scale_tkc_cluster(
    si: ServiceInstance, name: str, namespace: str, worker_count: int
) -> dict:
    """Scale TKC worker node count."""
    if worker_count < 1:
        raise ValueError(f"worker_count must be >= 1, got {worker_count}")
    api = _get_custom_objects_api(si, namespace)
    patch = {
        "spec": {
            "topology": {
                "workers": {
                    "machineDeployments": [{"name": "worker-pool", "replicas": worker_count}]
                }
            }
        }
    }
    api.patch_namespaced_custom_object(
        group=_TKC_GROUP, version=_TKC_VERSION,
        namespace=namespace, plural=_TKC_PLURAL, name=name, body=patch,
    )
    return {"name": name, "namespace": namespace, "worker_count": worker_count, "status": "scaling"}


def upgrade_tkc_cluster(
    si: ServiceInstance, name: str, namespace: str, k8s_version: str
) -> dict:
    """Upgrade TKC cluster K8s version."""
    api = _get_custom_objects_api(si, namespace)
    patch = {"spec": {"topology": {"version": k8s_version}}}
    api.patch_namespaced_custom_object(
        group=_TKC_GROUP, version=_TKC_VERSION,
        namespace=namespace, plural=_TKC_PLURAL, name=name, body=patch,
    )
    return {"name": name, "namespace": namespace, "new_version": k8s_version, "status": "upgrading"}


def _check_running_workloads(si: ServiceInstance, name: str, namespace: str) -> list[dict]:
    """Check for running Deployments/StatefulSets in the TKC cluster."""
    from vmware_vks.ops.kubeconfig import get_tkc_kubeconfig_str
    import kubernetes as k8s
    import tempfile

    try:
        kubeconfig_str = get_tkc_kubeconfig_str(si, name, namespace)
        with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as f:
            f.write(kubeconfig_str)
            tmpfile = f.name
        try:
            cfg = k8s.config.load_kube_config(config_file=tmpfile)
            api_client = k8s.client.ApiClient(configuration=cfg)
            apps_api = k8s.client.AppsV1Api(api_client)
            workloads = []
            for deploy in apps_api.list_deployment_for_all_namespaces().items:
                if deploy.status.ready_replicas and deploy.status.ready_replicas > 0:
                    workloads.append({
                        "kind": "Deployment",
                        "name": deploy.metadata.name,
                        "namespace": deploy.metadata.namespace,
                    })
            for ss in apps_api.list_stateful_set_for_all_namespaces().items:
                if ss.status.ready_replicas and ss.status.ready_replicas > 0:
                    workloads.append({
                        "kind": "StatefulSet",
                        "name": ss.metadata.name,
                        "namespace": ss.metadata.namespace,
                    })
            for ds in apps_api.list_daemon_set_for_all_namespaces().items:
                if ds.status and ds.status.number_ready and ds.status.number_ready > 0:
                    workloads.append({
                        "kind": "DaemonSet",
                        "name": ds.metadata.name,
                        "namespace": ds.metadata.namespace,
                        "ready": ds.status.number_ready,
                    })
            return workloads
        finally:
            Path(tmpfile).unlink(missing_ok=True)
    except Exception as e:
        _log.warning("Could not verify workloads in cluster '%s': %s", name, e)
        raise RuntimeError(
            f"Cannot verify workloads in TKC cluster '{name}': {e}. "
            "Use force=True to skip workload check."
        ) from e


def delete_tkc_cluster(
    si: ServiceInstance,
    name: str,
    namespace: str,
    confirmed: bool = False,
    dry_run: bool = True,
    force: bool = False,
) -> dict:
    """Delete a TKC cluster with workload guard.

    Guards: confirmed=True required; rejects if workloads running (unless force=True).
    """
    if not confirmed:
        raise ValueError(
            f"confirmed=True required to delete TKC cluster '{name}'."
        )

    if not force:
        workloads = _check_running_workloads(si, name, namespace)
        if workloads:
            raise RuntimeError(
                f"Cannot delete TKC cluster '{name}': "
                f"{len(workloads)} running workload(s) detected: "
                f"{[w['kind'] + '/' + w['name'] for w in workloads[:5]]}. "
                "Delete workloads first or use force=True."
            )

    if dry_run:
        return {
            "dry_run": True,
            "action": "delete_tkc_cluster",
            "name": name,
            "namespace": namespace,
            "warning": "This will permanently delete the TKC cluster.",
        }

    api = _get_custom_objects_api(si, namespace)
    api.delete_namespaced_custom_object(
        group=_TKC_GROUP, version=_TKC_VERSION,
        namespace=namespace, plural=_TKC_PLURAL, name=name,
    )
    return {"name": name, "namespace": namespace, "status": "deleting"}



"""Tests for supervisor ops (unit, mocked ServiceInstance)."""
from unittest.mock import MagicMock, patch
import pytest
from vmware_vks.ops.supervisor import (
    check_vks_compatibility,
    get_supervisor_status,
    list_supervisor_storage_policies,
)


def _mock_si(version="8.0.2"):
    si = MagicMock()
    si.content.about.version = version
    si.content.about.build = "21290409"
    si.content.sessionManager.currentSession.key = "session-123"
    si._stub.host = "vcenter.example.com:443"
    return si


def test_get_supervisor_status_returns_dict():
    si = _mock_si()
    with patch("vmware_vks.ops.supervisor._rest_get") as mock_get:
        mock_get.return_value = {
            "config_status": "RUNNING",
            "kubernetes_status": "READY",
            "api_server_cluster_endpoint": "192.168.1.10:6443",
        }
        result = get_supervisor_status(si, "domain-c1")
    assert result["kubernetes_status"] == "READY"
    assert "api_server_cluster_endpoint" in result


def test_check_vks_compatibility_v8():
    si = _mock_si("8.0.2")
    with patch("vmware_vks.ops.supervisor._rest_get") as mock_get:
        mock_get.return_value = []
        result = check_vks_compatibility(si)
    assert result["vcenter_version"] == "8.0.2"
    assert result["compatible"] is True


def test_check_vks_compatibility_v7_fails():
    si = _mock_si("7.0.3")
    with patch("vmware_vks.ops.supervisor._rest_get") as mock_get:
        mock_get.return_value = []
        result = check_vks_compatibility(si)
    assert result["compatible"] is False
    assert result["hint"] is not None


def test_list_supervisor_storage_policies_returns_list():
    si = _mock_si()
    with patch("vmware_vks.ops.supervisor._rest_get") as mock_get:
        mock_get.return_value = [
            {"storage_policy": "vsphere-storage", "compatible_clusters": ["domain-c1"]}
        ]
        result = list_supervisor_storage_policies(si)
    assert isinstance(result, list)
    assert result[0]["storage_policy"] == "vsphere-storage"



"""Configuration management for VMware VKS.

Loads targets and settings from YAML config file + environment variables.
Passwords are NEVER stored in config files — always via environment variables.
"""

from __future__ import annotations

import logging
import os
import stat
from dataclasses import dataclass, field
from pathlib import Path


import yaml
from dotenv import load_dotenv

CONFIG_DIR = Path.home() / ".vmware-vks"
CONFIG_FILE = CONFIG_DIR / "config.yaml"
ENV_FILE = CONFIG_DIR / ".env"

_log = logging.getLogger("vmware-vks.config")

load_dotenv(ENV_FILE)


def _check_env_permissions() -> None:
    if not ENV_FILE.exists():
        return
    try:
        mode = ENV_FILE.stat().st_mode
        if mode & (stat.S_IRWXG | stat.S_IRWXO):
            _log.warning(
                "Security warning: %s has permissions %s (should be 600). "
                "Run: chmod 600 %s",
                ENV_FILE,
                oct(stat.S_IMODE(mode)),
                ENV_FILE,
            )
    except OSError:
        pass


_check_env_permissions()


@dataclass(frozen=True)
class TargetConfig:
    """A vCenter connection target."""

    name: str
    host: str
    username: str
    port: int = 443
    verify_ssl: bool = False

    @property
    def password(self) -> str:
        env_key = f"VMWARE_VKS_{self.name.upper().replace('-', '_')}_PASSWORD"
        pw = os.environ.get(env_key, "")
        if not pw:
            raise OSError(
                f"Password not found. Set environment variable: {env_key}"
            )
        return pw


@dataclass(frozen=True)
class AppConfig:
    """Top-level application config."""

    targets: tuple[TargetConfig, ...] = ()

    def get_target(self, name: str) -> TargetConfig:
        for t in self.targets:
            if t.name == name:
                return t
        available = ", ".join(t.name for t in self.targets)
        raise KeyError(f"Target '{name}' not found. Available: {available}")

    @property
    def default_target(self) -> TargetConfig:
        if not self.targets:
            raise ValueError("No targets configured. Check config.yaml")
        return self.targets[0]


def load_config(config_path: Path | None = None) -> AppConfig:
    """Load config from YAML file."""
    path = config_path or CONFIG_FILE
    if not path.exists():
        raise FileNotFoundError(
            f"Config file not found: {path}\n"
            f"Copy config.example.yaml to {CONFIG_FILE} and edit it."
        )
    with open(path) as f:
        raw = yaml.safe_load(f) or {}
    targets = tuple(
        TargetConfig(
            name=t["name"],
            host=t["host"],
            username=t.get("username", "administrator@vsphere.local"),
            port=t.get("port", 443),
            verify_ssl=t.get("verify_ssl", False),
        )
        for t in raw.get("targets", [])
    )
    return AppConfig(targets=targets)



"""Supervisor layer operations (read-only).

Uses vCenter REST API via urllib (same session cookie as pyVmomi).
All functions take (si: ServiceInstance) as first argument.
"""
from __future__ import annotations

import json
import logging
import ssl
import urllib.error
import urllib.request
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance

_log = logging.getLogger("vmware-vks.ops.supervisor")

_MIN_VERSION = (8, 0, 0)


def _vcenter_host(si: ServiceInstance) -> str:
    return si._stub.host.split(":")[0]


def _rest_get(si: ServiceInstance, path: str) -> Any:
    """Authenticated REST GET using active pyVmomi session cookie."""
    host = _vcenter_host(si)
    session_id = si.content.sessionManager.currentSession.key
    url = f"https://{host}/api{path}"

    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    req = urllib.request.Request(url, headers={"vmware-api-session-id": session_id})
    try:
        with urllib.request.urlopen(req, context=ctx) as resp:  # nosec B310
            return json.loads(resp.read())
    except urllib.error.HTTPError as e:
        body = e.read().decode()
        raise RuntimeError(f"REST GET {path} failed ({e.code}): {body}") from e


def check_vks_compatibility(si: ServiceInstance) -> dict:
    """Check if this vCenter supports VKS (vSphere 8.x+)."""
    about = si.content.about
    version_str = about.version
    parts = tuple(int(x) for x in version_str.split(".")[:3])
    compatible = parts >= _MIN_VERSION

    try:
        clusters = _rest_get(si, "/vcenter/namespace-management/clusters")
    except Exception:
        clusters = []

    enabled = [c for c in clusters if c.get("config_status") == "RUNNING"]

    return {
        "compatible": compatible,
        "vcenter_version": version_str,
        "vcenter_build": about.build,
        "min_required_version": "8.0.0",
        "wcp_enabled_clusters": len(enabled),
        "wcp_clusters": [
            {"cluster": c.get("cluster"), "status": c.get("config_status")}
            for c in clusters
        ],
        "hint": None if compatible else "VKS requires vSphere 8.0+. Upgrade vCenter.",
    }


def get_supervisor_status(si: ServiceInstance, cluster_id: str) -> dict:
    """Get Supervisor Cluster status for a given compute cluster MoRef ID."""
    data = _rest_get(si, f"/vcenter/namespace-management/clusters/{cluster_id}")
    return {
        "cluster_id": cluster_id,
        "config_status": data.get("config_status"),
        "kubernetes_status": data.get("kubernetes_status"),
        "api_server_cluster_endpoint": data.get("api_server_cluster_endpoint"),
        "kubernetes_version": data.get("current_kubernetes_version"),
        "network_provider": data.get("network_provider"),
    }


def list_supervisor_storage_policies(si: ServiceInstance) -> list[dict]:
    """List storage policies compatible with Supervisor Namespaces."""
    data = _rest_get(si, "/vcenter/namespace-management/storage/storage-policies")
    return [
        {
            "storage_policy": item.get("storage_policy"),
            "compatible_clusters": item.get("compatible_clusters", []),
        }
        for item in (data if isinstance(data, list) else [])
    ]



"""Typer CLI for vmware-vks."""
from __future__ import annotations

import os
from pathlib import Path
from typing import Optional

import typer
from rich.console import Console
from rich.table import Table

app = typer.Typer(name="vmware-vks", help="vSphere with Tanzu (VKS) management CLI")
supervisor_app = typer.Typer(help="Supervisor cluster commands")
namespace_app = typer.Typer(help="Namespace commands")

app.add_typer(supervisor_app, name="supervisor")
app.add_typer(namespace_app, name="namespace")

tkc_app = typer.Typer(help="TanzuKubernetesCluster commands")
kubeconfig_app = typer.Typer(help="Kubeconfig commands")

app.add_typer(tkc_app, name="tkc")
app.add_typer(kubeconfig_app, name="kubeconfig")

console = Console()


def _get_si(target: str | None = None):
    from vmware_vks.config import load_config
    from vmware_vks.connection import ConnectionManager
    config_path = os.environ.get("VMWARE_VKS_CONFIG")
    config = load_config(Path(config_path) if config_path else None)
    mgr = ConnectionManager(config)
    return mgr.connect(target)


def _double_confirm(resource: str, resource_type: str = "resource") -> bool:
    console.print(f"[red]WARNING: You are about to delete {resource_type} '{resource}'.[/red]")
    typed = typer.prompt(f"Type '{resource}' to confirm")
    return typed == resource


@app.command("check")
def cmd_check(
    config: Optional[Path] = typer.Option(None, help="Path to config.yaml"),
):
    """Run pre-flight checks (config, passwords, vCenter version, WCP)."""
    from vmware_vks.doctor import run_doctor
    ok = run_doctor(config)
    raise typer.Exit(0 if ok else 1)


@supervisor_app.command("status")
def supervisor_status(
    cluster_id: str = typer.Argument(..., help="Compute cluster MoRef (e.g. domain-c1)"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Get Supervisor Cluster status."""
    from vmware_vks.ops.supervisor import get_supervisor_status
    si = _get_si(target)
    result = get_supervisor_status(si, cluster_id)
    for k, v in result.items():
        console.print(f"  [bold]{k}:[/bold] {v}")


@supervisor_app.command("storage-policies")
def supervisor_storage_policies(
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """List storage policies available for Namespaces."""
    from vmware_vks.ops.supervisor import list_supervisor_storage_policies
    si = _get_si(target)
    policies = list_supervisor_storage_policies(si)
    table = Table("Storage Policy", "Compatible Clusters")
    for p in policies:
        table.add_row(p["storage_policy"], str(p["compatible_clusters"]))
    console.print(table)


@namespace_app.command("list")
def namespace_list(
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """List all vSphere Namespaces."""
    from vmware_vks.ops.namespace import list_namespaces
    si = _get_si(target)
    nss = list_namespaces(si)
    table = Table("Namespace", "Status", "Description")
    for ns in nss:
        table.add_row(ns["namespace"], ns["config_status"], ns.get("description", ""))
    console.print(table)


@namespace_app.command("get")
def namespace_get(
    name: str = typer.Argument(...),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Get details for a single Namespace."""
    import json
    from vmware_vks.ops.namespace import get_namespace
    si = _get_si(target)
    result = get_namespace(si, name)
    console.print_json(json.dumps(result))


@namespace_app.command("create")
def namespace_create(
    name: str = typer.Argument(...),
    cluster_id: str = typer.Option(..., "--cluster", help="Supervisor cluster MoRef"),
    storage_policy: str = typer.Option(..., "--storage-policy"),
    cpu_limit: Optional[int] = typer.Option(None, "--cpu"),
    memory_mib: Optional[int] = typer.Option(None, "--memory"),
    description: str = typer.Option("", "--description"),
    apply: bool = typer.Option(False, "--apply", help="Apply (default: dry-run)"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Create a vSphere Namespace (dry-run by default, use --apply to create)."""
    import json
    from vmware_vks.ops.namespace import create_namespace
    si = _get_si(target)
    result = create_namespace(
        si, name=name, cluster_id=cluster_id, storage_policy=storage_policy,
        cpu_limit=cpu_limit, memory_limit_mib=memory_mib,
        description=description, dry_run=not apply,
    )
    console.print_json(json.dumps(result))


@namespace_app.command("update")
def namespace_update(
    name: str = typer.Argument(...),
    cpu_limit: Optional[int] = typer.Option(None, "--cpu"),
    memory_mib: Optional[int] = typer.Option(None, "--memory"),
    storage_policy: Optional[str] = typer.Option(None, "--storage-policy"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Update Namespace resource quotas or storage policy."""
    import json
    from vmware_vks.ops.namespace import update_namespace
    si = _get_si(target)
    result = update_namespace(si, name, cpu_limit=cpu_limit,
                              memory_limit_mib=memory_mib, storage_policy=storage_policy)
    console.print_json(json.dumps(result))


@namespace_app.command("delete")
def namespace_delete(
    name: str = typer.Argument(...),
    force: bool = typer.Option(False, "--force", help="Skip interactive confirm"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Delete a vSphere Namespace (rejects if TKC clusters exist inside)."""
    import json
    from vmware_vks.ops.namespace import delete_namespace

    si = _get_si(target)
    # Show dry-run first
    dry = delete_namespace(si, name, confirmed=True, dry_run=True)
    console.print_json(json.dumps(dry))

    if not force:
        if not _double_confirm(name, "namespace"):
            console.print("[yellow]Aborted.[/yellow]")
            raise typer.Exit(1)

    result = delete_namespace(si, name, confirmed=True, dry_run=False)
    console.print_json(json.dumps(result))


@namespace_app.command("vm-classes")
def namespace_vm_classes(
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """List available VM classes for TKC node sizing."""
    from vmware_vks.ops.namespace import list_vm_classes
    si = _get_si(target)
    classes = list_vm_classes(si)
    table = Table("ID", "CPU", "Memory (MiB)", "GPU")
    for c in classes:
        table.add_row(str(c["id"]), str(c["cpu_count"]), str(c["memory_mib"]), str(c["gpu_count"]))
    console.print(table)


# ---------------------------------------------------------------------------
# TKC commands
# ---------------------------------------------------------------------------

@tkc_app.command("list")
def tkc_list(
    namespace: Optional[str] = typer.Option(None, "-n", "--namespace"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """List TKC clusters (optionally filtered by namespace)."""
    from vmware_vks.ops.tkc import list_tkc_clusters
    si = _get_si(target)
    result = list_tkc_clusters(si, namespace=namespace)
    table = Table("Name", "Namespace", "Phase", "K8s Version")
    for c in result["clusters"]:
        table.add_row(c["name"], c["namespace"], c["phase"], c["k8s_version"])
    console.print(f"Total: {result['total']}")
    console.print(table)


@tkc_app.command("get")
def tkc_get(
    name: str = typer.Argument(...),
    namespace: str = typer.Option(..., "-n", "--namespace"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Get detailed info for a TKC cluster."""
    import json
    from vmware_vks.ops.tkc import get_tkc_cluster
    si = _get_si(target)
    result = get_tkc_cluster(si, name, namespace)
    console.print_json(json.dumps(result))


@tkc_app.command("versions")
def tkc_versions(
    namespace: str = typer.Option(..., "-n", "--namespace", help="vSphere Namespace"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """List available K8s versions for TKC clusters."""
    from vmware_vks.ops.tkc import get_tkc_available_versions
    si = _get_si(target)
    result = get_tkc_available_versions(si, namespace)
    if result.get("error"):
        console.print(f"[yellow]{result['error']}[/yellow]")
        console.print(f"[dim]{result.get('hint', '')}[/dim]")
        return
    table = Table("Version")
    for v in result["versions"]:
        table.add_row(v["version"])
    console.print(table)


@tkc_app.command("create")
def tkc_create(
    name: str = typer.Argument(...),
    namespace: str = typer.Option(..., "-n", "--namespace"),
    k8s_version: Optional[str] = typer.Option(None, "--version"),
    vm_class: Optional[str] = typer.Option(None, "--vm-class"),
    control_plane: int = typer.Option(1, "--control-plane"),
    workers: int = typer.Option(3, "--workers"),
    storage_policy: str = typer.Option("vsphere-storage", "--storage-policy"),
    apply: bool = typer.Option(False, "--apply", help="Apply (default: dry-run)"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Create a TKC cluster (dry-run by default, use --apply to create).

    Missing --version or --vm-class triggers interactive prompts.
    """
    import json
    from vmware_vks.ops.tkc import create_tkc_cluster, get_tkc_available_versions
    from vmware_vks.ops.namespace import list_vm_classes

    si = _get_si(target)

    # Interactive prompts for missing params
    if not k8s_version:
        versions_result = get_tkc_available_versions(si, namespace)
        version_choices = [v["version"] for v in versions_result.get("versions", [])]
        if version_choices:
            console.print(f"Available versions: {', '.join(version_choices[:5])}")
        k8s_version = typer.prompt("K8s version")

    if not vm_class:
        classes = list_vm_classes(si)
        class_choices = [c["id"] for c in classes if c.get("id")]
        if class_choices:
            console.print(f"Available VM classes: {', '.join(class_choices[:5])}")
        vm_class = typer.prompt("VM class")

    result = create_tkc_cluster(
        si, name=name, namespace=namespace, k8s_version=k8s_version,
        vm_class=vm_class, control_plane_count=control_plane,
        worker_count=workers, storage_class=storage_policy,
        dry_run=not apply,
    )
    console.print_json(json.dumps(result))


@tkc_app.command("scale")
def tkc_scale(
    name: str = typer.Argument(...),
    namespace: str = typer.Option(..., "-n", "--namespace"),
    workers: int = typer.Option(..., "--workers"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Scale TKC cluster worker node count."""
    import json
    from vmware_vks.ops.tkc import scale_tkc_cluster
    si = _get_si(target)
    result = scale_tkc_cluster(si, name, namespace, workers)
    console.print_json(json.dumps(result))


@tkc_app.command("upgrade")
def tkc_upgrade(
    name: str = typer.Argument(...),
    namespace: str = typer.Option(..., "-n", "--namespace"),
    version: str = typer.Option(..., "--version"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Upgrade TKC cluster to a new K8s version."""
    import json
    from vmware_vks.ops.tkc import upgrade_tkc_cluster
    si = _get_si(target)
    result = upgrade_tkc_cluster(si, name, namespace, version)
    console.print_json(json.dumps(result))


@tkc_app.command("delete")
def tkc_delete(
    name: str = typer.Argument(...),
    namespace: str = typer.Option(..., "-n", "--namespace"),
    force: bool = typer.Option(False, "--force"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Delete a TKC cluster (rejects if workloads running)."""
    import json
    from vmware_vks.ops.tkc import delete_tkc_cluster

    si = _get_si(target)

    # Show dry-run first
    dry = delete_tkc_cluster(si, name, namespace, confirmed=True, dry_run=True, force=force)
    console.print_json(json.dumps(dry))

    if not _double_confirm(name, "TKC cluster"):
        console.print("[yellow]Aborted.[/yellow]")
        raise typer.Exit(1)

    result = delete_tkc_cluster(si, name, namespace, confirmed=True, dry_run=False, force=force)
    console.print_json(json.dumps(result))


# ---------------------------------------------------------------------------
# Kubeconfig commands
# ---------------------------------------------------------------------------

@kubeconfig_app.command("supervisor")
def kubeconfig_supervisor(
    namespace: str = typer.Option(..., "-n", "--namespace"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Get Supervisor kubeconfig."""
    from vmware_vks.ops.kubeconfig import get_supervisor_kubeconfig_str
    si = _get_si(target)
    kc = get_supervisor_kubeconfig_str(si, namespace)
    console.print(kc)


@kubeconfig_app.command("get")
def kubeconfig_get(
    name: str = typer.Argument(...),
    namespace: str = typer.Option(..., "-n", "--namespace"),
    output: Optional[Path] = typer.Option(None, "--output", "-o"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Get TKC cluster kubeconfig."""
    import json
    from vmware_vks.ops.kubeconfig import write_kubeconfig
    si = _get_si(target)
    result = write_kubeconfig(si, name, namespace, output_path=output)
    if output:
        console.print(f"[green]Written to {output}[/green]")
    else:
        console.print(result.get("kubeconfig", ""))


# ---------------------------------------------------------------------------
# Harbor command
# ---------------------------------------------------------------------------

@app.command("harbor")
def harbor_info(
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """Get Harbor registry info."""
    import json
    from vmware_vks.ops.harbor import get_harbor_info
    si = _get_si(target)
    result = get_harbor_info(si)
    console.print_json(json.dumps(result))


# ---------------------------------------------------------------------------
# Storage command
# ---------------------------------------------------------------------------

@app.command("storage")
def storage_usage(
    namespace: str = typer.Option(..., "-n", "--namespace"),
    target: Optional[str] = typer.Option(None, "-t", "--target"),
):
    """List PVC storage usage for a Namespace."""
    import json
    from vmware_vks.ops.storage import list_namespace_storage_usage
    si = _get_si(target)
    result = list_namespace_storage_usage(si, namespace)
    console.print_json(json.dumps(result))



"""Tests for harbor ops."""
from unittest.mock import MagicMock, patch
import pytest
from vmware_vks.ops.harbor import get_harbor_info


def _mock_si():
    si = MagicMock()
    si.content.about.version = "8.0.2"
    si._stub.host = "vcenter.example.com:443"
    si.content.sessionManager.currentSession.key = "session-123"
    return si


def test_get_harbor_info_returns_dict():
    si = _mock_si()
    with patch("vmware_vks.ops.harbor._rest_get") as mock_get:
        mock_get.return_value = [
            {
                "id": "harbor-1",
                "ui_access_url": "https://harbor.example.com",
                "storage_used_MB": 1024,
                "health": {"status": "HEALTHY"},
            }
        ]
        result = get_harbor_info(si)

    assert isinstance(result, dict)
    assert "registries" in result
    assert len(result["registries"]) == 1
    assert result["registries"][0]["url"] == "https://harbor.example.com"
    assert result["registries"][0]["storage_used_mb"] == 1024
    assert result["registries"][0]["status"] == "HEALTHY"


def test_get_harbor_info_not_configured():
    si = _mock_si()
    with patch("vmware_vks.ops.harbor._rest_get") as mock_get:
        mock_get.side_effect = Exception("404 Not Found")
        result = get_harbor_info(si)

    assert isinstance(result, dict)
    assert "error" in result
    assert "404" in result["error"]
    assert "hint" in result



"""Kubeconfig retrieval for Supervisor and TKC clusters."""
from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance

_log = logging.getLogger("vmware-vks.ops.kubeconfig")


def get_supervisor_kubeconfig_str(si: ServiceInstance, namespace: str) -> str:
    """Get kubeconfig YAML string for Supervisor namespace."""
    from vmware_vks.k8s_connection import get_supervisor_kubeconfig_str as _get
    return _get(si, namespace)


def get_tkc_kubeconfig_str(si: ServiceInstance, cluster_name: str, namespace: str) -> str:
    """Get kubeconfig YAML string for a TKC cluster via Supervisor API."""
    import kubernetes as k8s
    import yaml as _yaml
    from vmware_vks.k8s_connection import get_k8s_client

    api_client = get_k8s_client(si, namespace)
    custom_api = k8s.client.CustomObjectsApi(api_client)

    cluster = custom_api.get_namespaced_custom_object(
        group="cluster.x-k8s.io", version="v1beta1",
        namespace=namespace, plural="clusters", name=cluster_name,
    )

    control_plane_endpoint = cluster.get("spec", {}).get("controlPlaneEndpoint", {})
    host = control_plane_endpoint.get("host", "")
    port = control_plane_endpoint.get("port", 6443)

    if not host:
        raise RuntimeError(
            f"TKC cluster '{cluster_name}' control plane endpoint not available. "
            "Is the cluster fully provisioned?"
        )

    token = si.content.sessionManager.currentSession.key
    kubeconfig = {
        "apiVersion": "v1",
        "kind": "Config",
        "clusters": [{"name": cluster_name, "cluster": {
            "server": f"https://{host}:{port}",
            "insecure-skip-tls-verify": True,
        }}],
        "users": [{"name": "vsphere-user", "user": {"token": token}}],
        "contexts": [{"name": f"{cluster_name}-context", "context": {
            "cluster": cluster_name, "user": "vsphere-user",
        }}],
        "current-context": f"{cluster_name}-context",
    }
    return _yaml.dump(kubeconfig)


def write_kubeconfig(
    si: ServiceInstance,
    cluster_name: str,
    namespace: str,
    output_path: Path | None = None,
) -> dict:
    """Write TKC kubeconfig to file or return as string."""
    kubeconfig_str = get_tkc_kubeconfig_str(si, cluster_name, namespace)
    if output_path:
        output_path.write_text(kubeconfig_str)
        output_path.chmod(0o600)
        return {"cluster": cluster_name, "written_to": str(output_path)}
    return {"cluster": cluster_name, "kubeconfig": kubeconfig_str}



"""Namespace storage usage (PVC list + usage stats)."""
from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pyVmomi.vim import ServiceInstance


def list_namespace_storage_usage(si: ServiceInstance, namespace: str) -> dict:
    """List PVCs and storage usage for a vSphere Namespace."""
    import kubernetes as k8s
    from vmware_vks.k8s_connection import get_k8s_client

    api_client = get_k8s_client(si, namespace)
    core_api = k8s.client.CoreV1Api(api_client)

    pvcs = core_api.list_namespaced_persistent_volume_claim(namespace=namespace)
    items = [
        {
            "name": pvc.metadata.name,
            "namespace": pvc.metadata.namespace,
            "status": pvc.status.phase,
            "capacity": pvc.status.capacity.get("storage") if pvc.status.capacity else None,
            "storage_class": pvc.spec.storage_class_name,
        }
        for pvc in pvcs.items
    ]
    return {"namespace": namespace, "pvc_count": len(items), "pvcs": items}



"""MCP server for VMware VKS (vSphere with Tanzu).

Exposes Supervisor and Namespace management via MCP stdio transport.

Tool categories
---------------
* **Read-only**: check_vks_compatibility, get_supervisor_status,
  list_supervisor_storage_policies, list_namespaces, get_namespace, list_vm_classes
* **Write** (dry_run=True by default): create_namespace, update_namespace
* **Write** (confirmed + TKC guard): delete_namespace

Security
--------
* Credentials loaded from ~/.vmware-vks/.env (chmod 600 recommended)
* stdio transport only — no network listener
* delete_namespace rejects if TKC clusters exist inside
* All write operations logged to ~/.vmware-vks/audit.log

Source: https://github.com/zw008/VMware-VKS
"""
from __future__ import annotations

import logging
import os
from pathlib import Path

from mcp.server.fastmcp import FastMCP

from vmware_vks.config import load_config
from vmware_vks.connection import ConnectionManager
from vmware_vks.notify.audit import AuditLogger
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("vmware-vks.mcp")

mcp = FastMCP("VMware VKS")
_audit = AuditLogger()

_conn_mgr: ConnectionManager | None = None


def _get_conn_mgr() -> ConnectionManager:
    global _conn_mgr
    if _conn_mgr is None:
        config_path = os.environ.get("VMWARE_VKS_CONFIG")
        config = load_config(Path(config_path) if config_path else None)
        _conn_mgr = ConnectionManager(config)
    return _conn_mgr


def _get_si(target: str | None = None):
    return _get_conn_mgr().connect(target)


# ---------------------------------------------------------------------------
# Supervisor tools
# ---------------------------------------------------------------------------

@mcp.tool()
def check_vks_compatibility(target: str | None = None) -> dict:
    """Check if this vCenter supports VKS (requires vSphere 8.x+).

    Returns: compatible (bool), vcenter_version, wcp_enabled_clusters, hint.
    Call this first before any VKS operations.
    """
    si = _get_si(target)
    from vmware_vks.ops import supervisor as _sup
    return _sup.check_vks_compatibility(si)


@mcp.tool()
def get_supervisor_status(cluster_id: str, target: str | None = None) -> dict:
    """Get Supervisor Cluster status.

    Args:
        cluster_id: Compute cluster MoRef ID (e.g. 'domain-c1').
        target: vCenter target name (uses default if not specified).

    Returns: config_status, kubernetes_status, api_server_endpoint, k8s_version.
    """
    si = _get_si(target)
    from vmware_vks.ops import supervisor as _sup
    return _sup.get_supervisor_status(si, cluster_id)


@mcp.tool()
def list_supervisor_storage_policies(target: str | None = None) -> list[dict]:
    """List storage policies available for Supervisor Namespaces.

    Returns list of storage policies with compatible cluster IDs.
    Use this to find valid storage_policy values before creating namespaces.
    """
    si = _get_si(target)
    from vmware_vks.ops import supervisor as _sup
    return _sup.list_supervisor_storage_policies(si)


# ---------------------------------------------------------------------------
# Namespace tools
# ---------------------------------------------------------------------------

@mcp.tool()
def list_namespaces(target: str | None = None) -> list[dict]:
    """List all vSphere Namespaces with status."""
    si = _get_si(target)
    from vmware_vks.ops import namespace as _ns
    return _ns.list_namespaces(si)


@mcp.tool()
def get_namespace(name: str, target: str | None = None) -> dict:
    """Get detailed information for a single vSphere Namespace.

    Args:
        name: Namespace name (e.g. 'dev', 'production').
        target: vCenter target name (uses default if not specified).
    """
    si = _get_si(target)
    from vmware_vks.ops import namespace as _ns
    return _ns.get_namespace(si, name)


@mcp.tool()
def create_namespace(
    name: str,
    cluster_id: str,
    storage_policy: str,
    cpu_limit: int | None = None,
    memory_limit_mib: int | None = None,
    description: str = "",
    dry_run: bool = True,
    target: str | None = None,
) -> dict:
    """Create a vSphere Namespace on a Supervisor Cluster.

    IMPORTANT: dry_run=True by default — set dry_run=False to actually create.

    Args:
        name: Namespace name (lowercase, no spaces).
        cluster_id: Supervisor cluster MoRef (use get_supervisor_status to find).
        storage_policy: Storage policy name (use list_supervisor_storage_policies).
        cpu_limit: CPU limit in MHz (optional).
        memory_limit_mib: Memory limit in MiB (optional).
        dry_run: Preview without creating (default: True).
    """
    si = _get_si(target)
    from vmware_vks.ops import namespace as _ns
    t = target or "default"
    params = {"cluster_id": cluster_id, "storage_policy": storage_policy}
    try:
        result = _ns.create_namespace(
            si, name=name, cluster_id=cluster_id, storage_policy=storage_policy,
            cpu_limit=cpu_limit, memory_limit_mib=memory_limit_mib,
            description=description, dry_run=dry_run,
        )
    except Exception as e:
        if not dry_run:
            _audit.log(
                target=t, operation="create_namespace",
                resource=name, parameters=params,
                result=f"failed: {e}",
            )
        raise
    if not dry_run:
        _audit.log(
            target=t, operation="create_namespace",
            resource=name, parameters=params,
            result="success",
        )
    return result


@mcp.tool()
def update_namespace(
    name: str,
    cpu_limit: int | None = None,
    memory_limit_mib: int | None = None,
    storage_policy: str | None = None,
    target: str | None = None,
) -> dict:
    """Update vSphere Namespace resource quotas or storage policy.

    Args:
        name: Namespace name.
        cpu_limit: New CPU limit in MHz (optional).
        memory_limit_mib: New memory limit in MiB (optional).
        storage_policy: New storage policy name (optional).
    """
    si = _get_si(target)
    from vmware_vks.ops import namespace as _ns
    t = target or "default"
    try:
        result = _ns.update_namespace(si, name, cpu_limit=cpu_limit,
                                      memory_limit_mib=memory_limit_mib,
                                      storage_policy=storage_policy)
    except Exception as e:
        _audit.log(target=t, operation="update_namespace",
                   resource=name, parameters={}, result=f"failed: {e}")
        raise
    _audit.log(target=t, operation="update_namespace",
               resource=name, parameters={}, result="success")
    return result


@mcp.tool()
def delete_namespace(
    name: str,
    confirmed: bool = False,
    dry_run: bool = True,
    target: str | None = None,
) -> dict:
    """Delete a vSphere Namespace.

    SAFETY: Rejects if TKC clusters exist inside. Delete TKC clusters first.
    IMPORTANT: dry_run=True by default — set dry_run=False AND confirmed=True to delete.

    Args:
        name: Namespace name to delete.
        confirmed: Must be True to proceed (safety gate).
        dry_run: Preview without deleting (default: True).
    """
    si = _get_si(target)
    from vmware_vks.ops import namespace as _ns
    t = target or "default"
    try:
        result = _ns.delete_namespace(si, name, confirmed=confirmed, dry_run=dry_run)
    except Exception as e:
        if not dry_run and confirmed:
            _audit.log(target=t, operation="delete_namespace",
                       resource=name, parameters={}, result=f"failed: {e}")
        raise
    if not dry_run and confirmed:
        _audit.log(target=t, operation="delete_namespace",
                   resource=name, parameters={}, result="success")
    return result


@mcp.tool()
def list_vm_classes(target: str | None = None) -> list[dict]:
    """List available VM classes for TKC node sizing.

    Returns list with id, cpu_count, memory_mib per VM class.
    Use the 'id' field when creating TKC clusters.
    """
    si = _get_si(target)
    from vmware_vks.ops import namespace as _ns
    return _ns.list_vm_classes(si)


# ---------------------------------------------------------------------------
# TKC tools
# ---------------------------------------------------------------------------

@mcp.tool()
def list_tkc_clusters(namespace: str | None = None, target: str | None = None) -> dict:
    """List TanzuKubernetesCluster (TKC) clusters.

    Args:
        namespace: vSphere Namespace to filter by (lists all if not specified).
        target: vCenter target name.

    Returns: total count and list of clusters with status and K8s version.
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    return _tkc.list_tkc_clusters(si, namespace=namespace)


@mcp.tool()
def get_tkc_cluster(name: str, namespace: str, target: str | None = None) -> dict:
    """Get detailed info for a single TKC cluster.

    Args:
        name: TKC cluster name.
        namespace: vSphere Namespace containing the cluster.
        target: vCenter target name.
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    return _tkc.get_tkc_cluster(si, name, namespace)


@mcp.tool()
def get_tkc_available_versions(namespace: str, target: str | None = None) -> dict:
    """List K8s versions available for new TKC clusters.

    Args:
        namespace: vSphere Namespace (used to connect to Supervisor).
        target: vCenter target name.
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    return _tkc.get_tkc_available_versions(si, namespace)


@mcp.tool()
def create_tkc_cluster(
    name: str,
    namespace: str,
    k8s_version: str,
    vm_class: str,
    control_plane_count: int = 1,
    worker_count: int = 3,
    storage_class: str = "vsphere-storage",
    dry_run: bool = True,
    target: str | None = None,
) -> dict:
    """Create a TanzuKubernetesCluster.

    IMPORTANT: dry_run=True by default — returns YAML plan. Set dry_run=False to apply.

    Workflow: call get_tkc_available_versions first to find valid k8s_version,
    call list_vm_classes to find valid vm_class.

    Args:
        name: Cluster name.
        namespace: vSphere Namespace.
        k8s_version: K8s version (e.g. 'v1.28.4+vmware.1').
        vm_class: VM class for nodes (e.g. 'best-effort-large').
        control_plane_count: 1 or 3.
        worker_count: Number of worker nodes (>= 1).
        storage_class: Storage class name.
        dry_run: Return YAML plan without applying (default: True).
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    t = target or "default"
    params = {"k8s_version": k8s_version, "vm_class": vm_class, "workers": worker_count}
    try:
        result = _tkc.create_tkc_cluster(
            si, name=name, namespace=namespace, k8s_version=k8s_version,
            vm_class=vm_class, control_plane_count=control_plane_count,
            worker_count=worker_count, storage_class=storage_class, dry_run=dry_run,
        )
    except Exception as e:
        if not dry_run:
            _audit.log(
                target=t, operation="create_tkc_cluster",
                resource=f"{namespace}/{name}",
                parameters=params,
                result=f"failed: {e}",
            )
        raise
    if not dry_run:
        _audit.log(
            target=t, operation="create_tkc_cluster",
            resource=f"{namespace}/{name}",
            parameters=params,
            result="success",
        )
    return result


@mcp.tool()
def scale_tkc_cluster(
    name: str, namespace: str, worker_count: int, target: str | None = None
) -> dict:
    """Scale TKC cluster worker node count.

    Args:
        name: TKC cluster name.
        namespace: vSphere Namespace.
        worker_count: New worker node count (>= 1).
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    t = target or "default"
    try:
        result = _tkc.scale_tkc_cluster(si, name, namespace, worker_count)
    except Exception as e:
        _audit.log(target=t, operation="scale_tkc_cluster",
                   resource=f"{namespace}/{name}", parameters={"worker_count": worker_count},
                   result=f"failed: {e}")
        raise
    _audit.log(target=t, operation="scale_tkc_cluster",
               resource=f"{namespace}/{name}", parameters={"worker_count": worker_count},
               result="success")
    return result


@mcp.tool()
def upgrade_tkc_cluster(
    name: str, namespace: str, k8s_version: str, target: str | None = None
) -> dict:
    """Upgrade TKC cluster to a new K8s version.

    Args:
        name: TKC cluster name.
        namespace: vSphere Namespace.
        k8s_version: Target K8s version (use get_tkc_available_versions to list).
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    t = target or "default"
    try:
        result = _tkc.upgrade_tkc_cluster(si, name, namespace, k8s_version)
    except Exception as e:
        _audit.log(target=t, operation="upgrade_tkc_cluster",
                   resource=f"{namespace}/{name}", parameters={"k8s_version": k8s_version},
                   result=f"failed: {e}")
        raise
    _audit.log(target=t, operation="upgrade_tkc_cluster",
               resource=f"{namespace}/{name}", parameters={"k8s_version": k8s_version},
               result="success")
    return result


@mcp.tool()
def delete_tkc_cluster(
    name: str,
    namespace: str,
    confirmed: bool = False,
    dry_run: bool = True,
    force: bool = False,
    target: str | None = None,
) -> dict:
    """Delete a TKC cluster.

    SAFETY: Rejects if Deployments/StatefulSets are running (unless force=True).
    IMPORTANT: dry_run=True by default — set dry_run=False AND confirmed=True to delete.

    Args:
        name: TKC cluster name.
        namespace: vSphere Namespace.
        confirmed: Must be True to proceed (safety gate).
        dry_run: Preview without deleting (default: True).
        force: Skip workload check (dangerous).
    """
    si = _get_si(target)
    from vmware_vks.ops import tkc as _tkc
    t = target or "default"
    try:
        result = _tkc.delete_tkc_cluster(
            si, name, namespace, confirmed=confirmed, dry_run=dry_run, force=force,
        )
    except Exception as e:
        if not dry_run and confirmed:
            _audit.log(target=t, operation="delete_tkc_cluster",
                       resource=f"{namespace}/{name}", parameters={"force": force},
                       result=f"failed: {e}")
        raise
    if not dry_run and confirmed:
        _audit.log(target=t, operation="delete_tkc_cluster",
                   resource=f"{namespace}/{name}", parameters={"force": force},
                   result="success")
    return result


# ---------------------------------------------------------------------------
# Access tools
# ---------------------------------------------------------------------------

@mcp.tool()
def get_supervisor_kubeconfig(namespace: str, target: str | None = None) -> dict:
    """Get kubeconfig for the Supervisor K8s API endpoint.

    Args:
        namespace: vSphere Namespace (context for the kubeconfig).
        target: vCenter target name.

    Returns: kubeconfig YAML string.
    """
    si = _get_si(target)
    from vmware_vks.ops import kubeconfig as _kc
    kc_str = _kc.get_supervisor_kubeconfig_str(si, namespace)
    return {"namespace": namespace, "kubeconfig": kc_str}


@mcp.tool()
def get_tkc_kubeconfig(
    name: str,
    namespace: str,
    output_path: str | None = None,
    target: str | None = None,
) -> dict:
    """Get kubeconfig for a TKC cluster.

    Args:
        name: TKC cluster name.
        namespace: vSphere Namespace.
        output_path: Write to file if provided (e.g. '~/.kube/my-cluster.yaml').
                     Returns kubeconfig string if not specified.
    """
    from pathlib import Path
    si = _get_si(target)
    from vmware_vks.ops import kubeconfig as _kc
    path = Path(output_path).expanduser() if output_path else None
    return _kc.write_kubeconfig(si, name, namespace, output_path=path)


@mcp.tool()
def get_harbor_info(target: str | None = None) -> dict:
    """Get embedded Harbor registry info (URL, storage usage, status).

    Returns registry URL, storage used, and health status.
    Returns error hint if Harbor is not enabled on this Supervisor.
    """
    si = _get_si(target)
    from vmware_vks.ops import harbor as _harbor
    return _harbor.get_harbor_info(si)


@mcp.tool()
def list_namespace_storage_usage(namespace: str, target: str | None = None) -> dict:
    """List PVCs and storage usage for a vSphere Namespace.

    Args:
        namespace: vSphere Namespace name.
        target: vCenter target name.
    """
    si = _get_si(target)
    from vmware_vks.ops import storage as _storage
    return _storage.list_namespace_storage_usage(si, namespace)


def main() -> None:
    logging.basicConfig(level=logging.INFO)
    mcp.run(transport="stdio")


if __name__ == "__main__":
    main()



"""Pre-flight diagnostics for vmware-vks."""
from __future__ import annotations

import logging
from pathlib import Path

from rich.console import Console
from rich.table import Table

_log = logging.getLogger("vmware-vks.doctor")
console = Console()


def run_doctor(config_path: Path | None = None) -> bool:
    """Run all pre-flight checks. Returns True if all pass."""
    from vmware_vks.config import CONFIG_FILE, ENV_FILE, load_config

    checks: list[tuple[str, bool, str]] = []

    # 1. Config file
    path = config_path or CONFIG_FILE
    if path.exists():
        checks.append(("Config file", True, str(path)))
    else:
        checks.append(("Config file", False, f"Not found: {path}. Copy config.example.yaml"))

    # 2. Load config
    config = None
    try:
        config = load_config(path)
        checks.append(("Config parse", True, f"{len(config.targets)} target(s)"))
    except Exception as e:
        checks.append(("Config parse", False, str(e)))

    # 3. Passwords
    if config:
        for t in config.targets:
            try:
                _ = t.password
                checks.append((f"Password ({t.name})", True, "Set"))
            except OSError as e:
                checks.append((f"Password ({t.name})", False, str(e)))

    # 4. vCenter reachable + version + WCP
    if config:
        for t in config.targets:
            try:
                from vmware_vks.connection import ConnectionManager
                mgr = ConnectionManager(config)
                si = mgr.connect(t.name)
                version = si.content.about.version
                checks.append((f"vCenter reachable ({t.name})", True, f"v{version}"))

                parts = tuple(int(x) for x in version.split(".")[:2])
                if parts >= (8, 0):
                    checks.append((f"vCenter version ({t.name})", True, f"{version} >= 8.0 ✓"))
                else:
                    checks.append((f"vCenter version ({t.name})", False, f"{version} < 8.0 (requires 8.x+)"))

                from vmware_vks.ops.supervisor import _rest_get
                clusters = _rest_get(si, "/vcenter/namespace-management/clusters")
                running = [c for c in clusters if c.get("config_status") == "RUNNING"]
                if running:
                    checks.append((f"WCP enabled ({t.name})", True, f"{len(running)} cluster(s) running"))
                else:
                    checks.append((f"WCP enabled ({t.name})", False, "No running Supervisor. Enable Workload Management in vCenter UI."))
            except Exception as e:
                checks.append((f"vCenter reachable ({t.name})", False, str(e)))

    # Print table
    table = Table(title="vmware-vks Doctor", show_header=True)
    table.add_column("Check", style="bold")
    table.add_column("Status")
    table.add_column("Detail")

    all_passed = True
    for name, passed, detail in checks:
        status = "[green]✓ PASS[/green]" if passed else "[red]✗ FAIL[/red]"
        table.add_row(name, status, detail)
        if not passed:
            all_passed = False

    console.print(table)
    return all_passed



"""Guard: vmware-vks must never contain pyVmomi VM lifecycle operations."""
import ast
import pathlib

FORBIDDEN_CALLS = {
    "PowerOn", "PowerOff", "Destroy", "Clone", "Relocate",
    "ReconfigVM", "MarkAsVirtualMachine", "RegisterVM",
}


def test_no_vm_lifecycle_ops():
    src_dir = pathlib.Path("vmware_vks")
    for py_file in src_dir.rglob("*.py"):
        source = py_file.read_text()
        tree = ast.parse(source)
        for node in ast.walk(tree):
            if isinstance(node, ast.Attribute) and node.attr in FORBIDDEN_CALLS:
                raise AssertionError(
                    f"{py_file}: Found forbidden VM lifecycle call '{node.attr}'. "
                    "vmware-vks must not modify VMs directly."
                )



"""Tests for namespace ops."""
from unittest.mock import MagicMock, patch
import pytest
from vmware_vks.ops.namespace import (
    list_namespaces,
    get_namespace,
    create_namespace,
    update_namespace,
    delete_namespace,
    list_vm_classes,
)


def _mock_si():
    si = MagicMock()
    si.content.about.version = "8.0.2"
    si._stub.host = "vcenter.example.com:443"
    si.content.sessionManager.currentSession.key = "session-123"
    return si


def test_list_namespaces_returns_list():
    si = _mock_si()
    with patch("vmware_vks.ops.namespace._rest_get") as mock_get:
        mock_get.return_value = [
            {"namespace": "dev", "config_status": "RUNNING", "description": ""}
        ]
        result = list_namespaces(si)
    assert isinstance(result, list)
    assert result[0]["namespace"] == "dev"


def test_get_namespace_returns_dict():
    si = _mock_si()
    with patch("vmware_vks.ops.namespace._rest_get") as mock_get:
        mock_get.return_value = {
            "namespace": "dev",
            "config_status": "RUNNING",
            "resource_spec": {"cpu_limit": 4000, "memory_limit": 8192},
        }
        result = get_namespace(si, "dev")
    assert result["namespace"] == "dev"


def test_create_namespace_dry_run():
    si = _mock_si()
    result = create_namespace(
        si, name="dev", cluster_id="domain-c1",
        storage_policy="vsphere-storage", dry_run=True,
    )
    assert result["dry_run"] is True
    assert result["action"] == "create_namespace"


def test_delete_namespace_requires_confirmed():
    si = _mock_si()
    with pytest.raises(ValueError, match="confirmed=True"):
        delete_namespace(si, "dev", confirmed=False)


def test_delete_namespace_rejects_if_tkc_exists():
    si = _mock_si()
    with patch("vmware_vks.ops.namespace._list_tkc_in_namespace") as mock_tkc:
        mock_tkc.return_value = ["cluster-a"]
        with pytest.raises(RuntimeError, match="cluster-a"):
            delete_namespace(si, "dev", confirmed=True)


def test_delete_namespace_dry_run():
    si = _mock_si()
    with patch("vmware_vks.ops.namespace._list_tkc_in_namespace") as mock_tkc:
        mock_tkc.return_value = []
        result = delete_namespace(si, "dev", confirmed=True, dry_run=True)
    assert result["dry_run"] is True


def test_list_vm_classes_returns_list():
    si = _mock_si()
    with patch("vmware_vks.ops.namespace._rest_get") as mock_get:
        mock_get.return_value = [
            {"id": "best-effort-large", "cpu_count": 4, "memory_mib": 8192}
        ]
        result = list_vm_classes(si)
    assert result[0]["id"] == "best-effort-large"



"""Audit logger — writes JSON Lines to ~/.vmware-vks/audit.log."""
from __future__ import annotations

import json
import logging
from datetime import datetime, timezone
from pathlib import Path

_AUDIT_DIR = Path.home() / ".vmware-vks"
_AUDIT_FILE = _AUDIT_DIR / "audit.log"
_log = logging.getLogger("vmware-vks.audit")


class AuditLogger:
    def __init__(self, log_file: Path = _AUDIT_FILE) -> None:
        self._file = log_file
        self._file.parent.mkdir(parents=True, exist_ok=True)

    def log(
        self,
        target: str,
        operation: str,
        resource: str,
        parameters: dict,
        result: str,
        user: str = "",
    ) -> None:
        entry = {
            "timestamp": datetime.now(timezone.utc).isoformat(),
            "target": target,
            "operation": operation,
            "resource": resource,
            "parameters": parameters,
            "result": result,
            "user": user,
        }
        try:
            with open(self._file, "a") as f:
                f.write(json.dumps(entry) + "\n")
        except OSError as e:
            _log.warning("Failed to write audit log: %s", e)



"""Tests for kubeconfig ops."""
from unittest.mock import MagicMock, patch
import pytest
from vmware_vks.ops.kubeconfig import (
    get_supervisor_kubeconfig_str,
    get_tkc_kubeconfig_str,
    write_kubeconfig,
)


def _mock_si():
    si = MagicMock()
    si.content.about.version = "8.0.2"
    si._stub.host = "vcenter.example.com:443"
    si.content.sessionManager.currentSession.key = "session-123"
    return si


def test_get_supervisor_kubeconfig_returns_string():
    si = _mock_si()
    with patch("vmware_vks.ops.kubeconfig.get_supervisor_kubeconfig_str") as mock_get:
        # Since the module-level function delegates to k8s_connection,
        # we patch at the import target to verify the delegation.
        pass
    # Patch the inner import instead
    with patch("vmware_vks.k8s_connection.get_supervisor_kubeconfig_str") as mock_inner:
        mock_inner.return_value = "apiVersion: v1\nkind: Config\nclusters: []\n"
        result = get_supervisor_kubeconfig_str(si, "dev")
    assert isinstance(result, str)
    assert "apiVersion" in result


def test_get_tkc_kubeconfig_returns_string():
    si = _mock_si()
    mock_api_client = MagicMock()
    mock_custom_api = MagicMock()
    mock_custom_api.get_namespaced_custom_object.return_value = {
        "spec": {
            "controlPlaneEndpoint": {
                "host": "10.0.0.100",
                "port": 6443,
            }
        }
    }

    with patch("vmware_vks.k8s_connection.get_k8s_client", return_value=mock_api_client):
        with patch("kubernetes.client.CustomObjectsApi", return_value=mock_custom_api):
            result = get_tkc_kubeconfig_str(si, "my-cluster", "dev")

    assert isinstance(result, str)
    assert "my-cluster" in result
    assert "https://10.0.0.100:6443" in result
    assert "session-123" in result


def test_write_kubeconfig_to_file(tmp_path):
    si = _mock_si()
    output_file = tmp_path / "kubeconfig.yaml"
    fake_kubeconfig = "apiVersion: v1\nkind: Config\nclusters: []\n"

    with patch("vmware_vks.ops.kubeconfig.get_tkc_kubeconfig_str", return_value=fake_kubeconfig):
        result = write_kubeconfig(si, "my-cluster", "dev", output_path=output_file)

    assert output_file.exists()
    assert output_file.read_text() == fake_kubeconfig
    # Verify file permissions are 0o600
    assert oct(output_file.stat().st_mode & 0o777) == oct(0o600)
    assert result["cluster"] == "my-cluster"
    assert result["written_to"] == str(output_file)



"""Tests for AuditLogger."""
import json
from unittest.mock import patch
import pytest
from vmware_vks.notify.audit import AuditLogger


def test_audit_log_writes_json_line(tmp_path):
    log_file = tmp_path / "audit.log"
    logger = AuditLogger(log_file=log_file)

    logger.log(
        target="vcenter.example.com",
        operation="create_namespace",
        resource="dev",
        parameters={"cluster_id": "domain-c1"},
        result="success",
        user="admin@vsphere.local",
    )

    content = log_file.read_text().strip()
    entry = json.loads(content)
    assert entry["target"] == "vcenter.example.com"
    assert entry["operation"] == "create_namespace"
    assert entry["resource"] == "dev"
    assert entry["parameters"] == {"cluster_id": "domain-c1"}
    assert entry["result"] == "success"
    assert entry["user"] == "admin@vsphere.local"
    assert "timestamp" in entry


def test_audit_log_creates_directory(tmp_path):
    nested_dir = tmp_path / "sub" / "dir"
    log_file = nested_dir / "audit.log"

    # Directory should not exist yet
    assert not nested_dir.exists()

    logger = AuditLogger(log_file=log_file)

    # Directory should be created by __init__
    assert nested_dir.exists()

    logger.log(
        target="vcenter.example.com",
        operation="test",
        resource="test-res",
        parameters={},
        result="ok",
    )
    assert log_file.exists()


def test_audit_log_appends(tmp_path):
    log_file = tmp_path / "audit.log"
    logger = AuditLogger(log_file=log_file)

    logger.log(
        target="vc1",
        operation="op1",
        resource="res1",
        parameters={},
        result="success",
    )
    logger.log(
        target="vc2",
        operation="op2",
        resource="res2",
        parameters={"key": "val"},
        result="failure",
    )

    lines = log_file.read_text().strip().split("\n")
    assert len(lines) == 2

    entry1 = json.loads(lines[0])
    entry2 = json.loads(lines[1])
    assert entry1["target"] == "vc1"
    assert entry2["target"] == "vc2"
    assert entry2["result"] == "failure"
